@@ -1,7 +1,10 @@
-# $Id: Table.pm,v 1.7 1996/11/25 22:04:43 rik Exp $
+# $Id: Table.pm,v 1.10 1997/05/05 21:43:05 rik Exp $
 
-require Net::NISPlus::Object;
-require Net::NISPlus::Entry;
+BEGIN
+{
+  require Net::NISPlus::Object;
+  require Net::NISPlus::Entry;
+};
 
 package Net::NISPlus::Table;
 
@@ -17,7 +20,39 @@ sub new
     my($type) = Net::NISPlus::obj_type($_);
     if (defined($type) && $type == &Net::NISPlus::TABLE_OBJ)
     {
+Debug("setting full_path to $_\n");
       $self->{'full_path'} = $_;
+      $self->{'cookie'} = '';
+      last;
+    }
+  };
+
+  return 0 unless $self->{'full_path'};
+
+  bless $self;
+}
+
+sub create
+{
+  my($name) = shift;
+  my($path) = shift;
+  my($self) = {};
+  my($ret);
+  
+  $ret = Net::NISPlus::nis_add($path, @_);
+  if ($ret != 0)
+  {
+    Warning("create error: %s". niserror($ret));
+    return;
+  }
+
+  foreach (Net::NISPlus::nis_getnames($path))
+  {
+    my($type) = Net::NISPlus::obj_type($_);
+    if (defined($type) && $type == &Net::NISPlus::TABLE_OBJ)
+    {
+      $self->{'full_path'} = $_;
+      $self->{'cookie'} = '';
       last;
     }
   };
@@ -30,38 +65,14 @@ sub lookup
   my($me) = shift;
   my($ret, @res);
 
-  my($colnames, @colnames);
-  my($srchstring);
-  $colnames = $me->colnames;
-  @colnames = $me->colnames;
+  my($srchstring) = $me->indexedName(shift);
+  my(@colnames) = $me->colnames;
 
-  if ($#_ == 0)
-  {
-    $srchstring = shift;
-# ensure the full path is added
-    $srchstring =~ s/,.*/",".$me->{'full_path'}/e;
-  }
-  else
-  {
-    my(%srch) = @_;
-    $srchstring = "[";
-
-    foreach $key (keys %srch)
-    {
-      die "$key does not exist in $me->{'full_path'}\n"
-        unless defined($colnames->{$key});
-      $srchstring .= "," unless length($srchstring) == 1;
-      $srchstring .= "$key=$srch{$key}";
-    }
-
-    $srchstring .= "],$me->{'full_path'}";
-  }
-
-#print "lookup up $srchstring\n" if $Net::NISPlus::Debug;
-  ($ret, @res) = Net::NISPlus::entry_list($srchstring);
+  Debug("lookup looking up $srchstring\n");
+  ($ret, @res) = Net::NISPlus::entry_list($srchstring, undef);
   if ($ret != 0)
   {
-    warn("lookup $srchstring error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("lookup $srchstring error: ", niserror($ret));
     return ();
   }
   else
@@ -85,11 +96,10 @@ sub list
   my($retobj) = shift;
   my($ret, @res);
 
-  ($ret, @res) = Net::NISPlus::entry_list($me->{'full_path'},
-    $retobj ? $me : undef);
+  ($ret, @res) = Net::NISPlus::entry_list($me->fullPath, $retobj ? $me : undef);
   if ($ret != 0)
   {
-    warn("list error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("list error: ", niserror($ret), "\n");
     return ();
   }
   else
@@ -150,10 +160,10 @@ sub colnames
 
   if (!defined($me->{'colnames'}))
   {
-    ($ret, $res) = Net::NISPlus::table_info($me->{'full_path'});
+    ($ret, $res) = Net::NISPlus::table_info($me->fullPath);
     if ($ret != 0)
     {
-      warn("colnames error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+      Warning("colnames error: ", niserror($ret), "\n");
       return ();
     }
     else
@@ -175,10 +185,10 @@ sub setinfo
   my($info) = shift;
   my($ret, $res);
 
-  ($ret, $res) = Net::NISPlus::table_setinfo($me->{'full_path'}, $info);
+  ($ret, $res) = Net::NISPlus::table_setinfo($me->fullPath, $info);
   if ($ret != 0)
   {
-    warn("setinfo error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("setinfo error: ", niserror($ret), "\n");
     return ();
   }
   else
@@ -192,10 +202,10 @@ sub info
   my($me) = shift;
   my($ret, $res);
 
-  ($ret, $res) = Net::NISPlus::table_info($me->{'full_path'});
+  ($ret, $res) = Net::NISPlus::table_info($me->fullPath);
   if ($ret != 0)
   {
-    warn("info error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("info error: ", niserror($ret), "\n");
     return ();
   }
   else
@@ -222,7 +232,10 @@ sub add
 {
   my($ret, $res);
   my($me) = shift;
-  if (ref($_[0]) == "ARRAY")
+
+  my %info = %{info($me)};
+
+  if (ref($_[0]) eq "ARRAY")
   {
     my($names) = shift;
     foreach $data (@_)
@@ -230,22 +243,24 @@ sub add
       my(%data);
       foreach $name ($[..$#{@$names})
       {
-print " setting $names->[$name] to $data->[$name]\n" if $Net::NISPlus::Debug;
+        Debug(" setting $names->[$name] to $data->[$name]\n");
         $data{$names->[$name]} = $data->[$name];
       }
-print "adding\n" if $Net::NISPlus::Debug;
-      ($ret, $res) = Net::NISPlus::nis_add_entry($me->{'full_path'}, \%data);
+      Debug("adding\n");
+      ($ret, $res) = Net::NISPlus::nis_add_entry($me->fullPath, \%data,
+        $info{'owner'}, $info{'group'}, $info{'access'}, $info{'ttl'});
     }
   }
   else
   {
     my(%data) = @_;
-    ($ret, $res) = Net::NISPlus::nis_add_entry($me->{'full_path'}, \%data);
+    ($ret, $res) = Net::NISPlus::nis_add_entry($me->fullPath, \%data,
+        $info{'owner'}, $info{'group'}, $info{'access'}, $info{'ttl'});
   }
 
   if ($ret != 0)
   {
-    warn("add error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("add error: ", niserror($ret), "\n");
     return ();
   }
   else { return $res; }
@@ -256,7 +271,7 @@ print "adding\n" if $Net::NISPlus::Debug;
 Add an entry to the table, setting the info variable as we go.  Any columns
 not specified will be set to null strings.
 
-$table->add([key1, key2],
+$table->addinfo([key1, key2],
   ['values' => [ 'value1', 'value2' ],
    'access' => access,
    'domain' => domain,
@@ -277,11 +292,11 @@ sub addinfo
     my(%data);
     foreach $name ($[..$#{@$names})
     {
-print " setting $names->[$name] to $data->{'values'}->[$name]\n" if $Net::NISPlus::Debug;
+      Debug(" setting $names->[$name] to $data->{'values'}->[$name]\n");
       $data{$names->[$name]} = $data->{'values'}->[$name];
     }
-print "adding ($me->{'full_path'})\n" if $Net::NISPlus::Debug;
-    ($ret, $res) = Net::NISPlus::nis_add_entry($me->{'full_path'},
+    Debug("adding (%s)\n", $me->fullPath);
+    ($ret, $res) = Net::NISPlus::nis_add_entry($me->fullPath,
       \%data,
       $data->{'owner'},
       $data->{'group'},
@@ -292,7 +307,7 @@ print "adding ($me->{'full_path'})\n" if $Net::NISPlus::Debug;
 
   if ($ret != 0)
   {
-    warn("add error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("add error: ", niserror($ret), "\n");
     return ();
   }
   else { return $res; }
@@ -304,24 +319,29 @@ Remove a single entry from the table.  If the key/value pairs match
 more that one entry, an error occurs, and no entries are removed.  Use
 removem to remove multiple entries with a single command.
 
-$table->remove('key1' => 'value1', 'key2' => 'value2');
+$table->remove({'key1' => 'value1', 'key2' => 'value2'});
+
+or
+
+$table->remove("[key1=value1,key2=value2]");
+
+If you specify the table name in the indexed name form, it will be
+removed and replaced with the full name determined when the table
+object was created (and accessible with $table->fullName);
 
 =cut
 
 sub remove
 {
-  my($me, %data) = @_;
+  my($me) = shift;
   my($ret);
 
-  my($name) = "[";
-  foreach (keys %data) { $name .= "$_=$data{$_},"; }
-  $name =~ s/,$//;
-  $name .= "],$me->{'full_path'}";
-print "name=|$name|\n" if $Net::NISPlus::Debug;
+  my($name) = $me->indexedName(shift);
+  Debug("name=|$name|\n");
   ($ret) = Net::NISPlus::nis_remove_entry($name, 0);
   if ($ret != 0)
   {
-    warn("remove error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("remove error: ", niserror($ret), "\n");
     return 0;
   }
   else
@@ -333,26 +353,31 @@ print "name=|$name|\n" if $Net::NISPlus::Debug;
 =head2 removem
 
 Remove one or more entries from the table. All entries which match the
-key/value pairs will be removed. Use removem to remove a single entry.
+key/value pairs will be removed. Use remove to remove a single entry.
 
-$table->removem('key1' => 'value1', 'key2' => 'value2');
+$table->removem({'key1' => 'value1', 'key2' => 'value2'});
+
+or
+
+$table->removem("[key1=value1,key2=value2]");
+
+If you specify the table name in the indexed name form, it will be
+removed and replaced with the full name determined when the table
+object was created (and accessible with $table->fullName);
 
 =cut
 
 sub removem
 {
-  my($me, %data) = @_;
+  my($me) = shift;
   my($ret);
 
-  my($name) = "[";
-  foreach (keys %data) { $name .= "$_=$data{$_},"; }
-  $name =~ s/,$//;
-  $name .= "],$me->{'full_path'}";
-print "name=|$name|\n" if $Net::NISPlus::Debug;
+  my($name) = $me->indexedName(shift);
+  Debug("name=|$name|\n");
   ($ret) = Net::NISPlus::nis_remove_entry($name, 1);
   if ($ret != 0)
   {
-    warn("removem error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("removem error: ", niserror($ret), "\n");
     return 0;
   }
   else
@@ -374,11 +399,11 @@ sub clear
   my($me) = @_;
   my($ret);
 
-  ($ret) = Net::NISPlus::nis_remove_entry($me->{'full_path'},
+  ($ret) = Net::NISPlus::nis_remove_entry($me->fullPath,
     &Net::NISPlus::REM_MULTIPLE);
   if ($ret != 0)
   {
-    warn("clear error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("clear error: ", niserror($ret), "\n");
     return 0;
   }
   else
@@ -387,20 +412,32 @@ sub clear
   }
 }
 
+=head2 modify
+
+Change fields in a table entry.
+
+$table->modify({'key1' => 'value1', 'key2' => 'value2'}, {'key3' => 'newvalue3'});
+
+or
+
+$table->modify("[key1=value1,key2=value2]", {'key3' => 'newvalue3'});
+
+If you specify the table name in the indexed name form, it will be
+removed and replaced with the full name determined when the table
+object was created (and accessible with $table->fullName);
+
+=cut
+
 sub modify
 {
   my($me, $search, $replace) = @_;
   my($ret);
 
-  my($name) = "[";
-  foreach (keys %{$search}) { $name .= "$_=$search->{$_},"; }
-  $name =~ s/,$//;
-  $name .= "],$me->{'full_path'}";
-print "name=|$name|\n" if $Net::NISPlus::Debug;
+  my($name) = $me->indexedName($search);
   ($ret) = Net::NISPlus::nis_modify_entry($name, $replace, 0);
   if ($ret != 0)
   {
-    warn("modify error: ", Net::NISPlus::nis_sperrno($ret), " ($ret)\n");
+    Warning("modify error: ", niserror($ret), "\n");
     return 0;
   }
   else
@@ -409,6 +446,56 @@ print "name=|$name|\n" if $Net::NISPlus::Debug;
   }
 }
 
+=head2 first_entry
+
+first_entry retrieves the first entry in the table.  Data is returned
+in an array.
+
+@fields = $table->first_entry();
+
+=cut
+
+sub first_entry
+{
+  my($me) = @_;
+  my($ret, $cookie, $res);
+
+  ($ret, $cookie, $res) = Net::NISPlus::nis_first_entry($me->fullPath);
+  if ($ret != 0) {
+    Warning("first_entry error: ", niserror($ret), "\n");
+    return ();
+  } else {
+    $me->{'cookie'} = $cookie;
+    return @{$res};
+  }
+}
+
+=head2 next_entry
+
+next_entry successively returns the next entry in the table.
+first_entry should be called before next_entry.  Data is returned in
+an array.
+
+@fields = $table->next_entry();
+
+=cut
+
+sub next_entry
+{
+  my($me) = @_;
+  my($ret, $cookie, $res);
+
+  ($ret, $cookie, $res) = Net::NISPlus::nis_next_entry($me->fullPath,
+						       $me->{'cookie'});
+  if ($ret != 0) {
+    Warning("next_entry error: ", niserror($ret), "\n");
+    return ();
+  } else {
+    $me->{'cookie'} = $cookie;
+    return @{$res};
+  }
+}
+
 sub chmod
 {
 }
@@ -421,5 +508,73 @@ sub DESTROY
 {
 }
 
+# takes either a string of the form [a=b,c=d] or a hash reference of the form
+# { 'a'=>'b', 'c'=>'d' } and returns a canonical indexed name.
+sub indexedName
+{
+  my($me, $arg) = @_;
+  my($name);
+# if the argument is a hashref, then we build the indexed name from the hash.  
+  if (ref($arg) eq "hash")
+  {
+    $name = "[";
+    foreach $key (keys %$arg)
+    {
+      die "$key does not exist in ".$me->fullPath."\n" unless $me->isColname($key);
+      $name .= "," unless length($name) == 1;
+      $name .= "$key=$arg->{$key}";
+    }
+    $name .= "]";
+  }
+  else
+# if the argument is not a hashref, then it should be a string
+  {
+    $name = $arg;
+# if the user specifies a table name, it will be deleted and replaced
+# with the full path
+    $name =~ s/,.*//;
+  }
+  $name .= ",".$me->fullPath;
+}
+
+sub fullPath
+{
+  my($me) = shift;
+
+  $me->{'full_path'};
+}
+
+=head2 isColname
+
+returns TRUE if the given argument is a valid column name for the
+table
+
+$table->isColname("name");
+
+=cut
+
+sub isColname
+{
+  my($me, $val) = @_;
+  my($colnames);
+  $colnames = $me->colnames;
+  return exists $colnames->{$val};
+}
+
+sub niserror
+{
+  my($err) = shift;
+  Net::NISPlus::nis_sperrno($err)." ($err)";
+}
+
+sub Warning
+{
+  Net::NISPlus::prwarning(@_);
+}
+
+sub Debug
+{
+  Net::NISPlus::prdebug(@_);
+}
 1;
 __END__
